package com.yem.ensign.common;

import kd.bos.exception.KDBizException;

import java.math.BigDecimal;
import java.text.NumberFormat;


/**
 * BigDecimal工具类
 *
 */
public class BigDecimalUtils {

	public static BigDecimal ZERO = new BigDecimal("0.00");
	public static BigDecimal ZERO_POINT_SIX = new BigDecimal("0.6");
	public static BigDecimal ONE = new BigDecimal("1.00");
	public static BigDecimal ONE_HANDRED = new BigDecimal("100");
	public static BigDecimal ONE_THOUSAND = new BigDecimal("1000");
	public static BigDecimal ONE_MILLION = new BigDecimal("1000000");
	public static BigDecimal ONE_HANDRED_TEN = new BigDecimal("110");
	public static BigDecimal TEN_THOUSAND = new BigDecimal("10000");
	public static BigDecimal MAX = new BigDecimal("9223372036854775807");

	/***
	 * Object对象转换为BigDecimal对象
	 * @param obj 转换前Object对象
	 * @return 转换后BigDecimal对象
	 */
	public static BigDecimal toBigDecimal(Object obj){
		try{
			if(obj == null || obj.toString() == null){
				return BigDecimalUtils.ZERO;
			}else if(obj instanceof BigDecimal){
				return (BigDecimal)obj;
			}else if(obj instanceof Integer){
				return new BigDecimal(((Integer)obj).toString());
			}else if(obj instanceof Long){
				return new BigDecimal(((Long)obj).toString());
			}else if(obj instanceof Double){
				return new BigDecimal(((Double)obj).doubleValue());
			}else{
				String str = obj.toString().trim();
				if(str.toLowerCase().indexOf("e") > -1){
					return new BigDecimal(str);
				}else if(str.matches("^[+-]?\\d+[\\.\\d]?\\d*+$")){
					return new BigDecimal(str);
				}else{
					return BigDecimalUtils.ZERO;
				}
			}
		}catch(NumberFormatException e){
			e.printStackTrace();
			throw new KDBizException("Object对象转换BigDecimal格式出错：" + e.getMessage());
		}
    }
	
	/***
	 * 除法
	 * @param  a 被除数
	 * @param  b 除数
	 * @param scale 精度
	 * @return 除法BigDecimal对象结果
	 */
	public static BigDecimal div(Object a, Object b,int scale) {
		if (a == null || b == null){
			return ZERO;
		}
		BigDecimal aVal = toBigDecimal(a);
		BigDecimal bVal = toBigDecimal(b);
		return aVal.divide(bVal, scale, BigDecimal.ROUND_HALF_UP);
	}

	/***
	 * 除法
	 * @param  a 被除数
	 * @param  b 除数
	 * @param scale 精度
	 * @param roundingMode 进位方式
	 * @return 除法BigDecimal对象结果
	 */
	public static BigDecimal div(Object a, Object b,int scale,int roundingMode) {
		if (a == null || b == null){
			return ZERO;
		}
		BigDecimal aVal = toBigDecimal(a);
		BigDecimal bVal = toBigDecimal(b);
		return aVal.divide(bVal, scale, roundingMode);
	}

	/***
	 * 除法
	 * @param aVal 被除数
	 * @param bVal 除数
	 * @param scale 精度
	 * @return 除法BigDecimal对象结果
	 */
	public static BigDecimal div(BigDecimal aVal, BigDecimal bVal,int scale) {
		if (aVal == null || aVal.compareTo(ZERO)==0 || bVal == null || bVal.compareTo(ZERO)==0){
			return ZERO;
		}
		return aVal.divide(bVal, scale, BigDecimal.ROUND_HALF_UP);

	}

	/***
	 * 除法
	 * @param aVal 被除数
	 * @param bVal 除数
	 * @param scale 精度
	 * @param roundingMode 进位方式
	 * @return 除法BigDecimal对象结果
	 */
	public static BigDecimal div(BigDecimal aVal, BigDecimal bVal,int scale,int roundingMode) {
		if (aVal == null || aVal.compareTo(ZERO)==0 || bVal == null || bVal.compareTo(ZERO)==0){
			return ZERO;
		}
		return aVal.divide(bVal, scale, roundingMode);

	}

	/***
	 * 乘法
	 * @param a 被乘数
	 * @param b 乘数
	 * @param scale 精度
	 * @return 乘法BigDecimal对象结果
	 */
	public static BigDecimal multiply(Object a, Object b,int scale) {
		if (a == null || b == null){
			return ZERO;
		}
		BigDecimal aVal = toBigDecimal(a);
		BigDecimal bVal = toBigDecimal(b);
		return aVal.multiply(bVal).setScale(scale,BigDecimal.ROUND_HALF_UP);
	}

	/***
	 * 乘法
	 * @param a 被乘数
	 * @param b 乘数
	 * @param scale 精度
	 * @param roundingMode 进位方式
	 * @return 乘法BigDecimal对象结果
	 */
	public static BigDecimal multiply(Object a, Object b,int scale,int roundingMode) {
		if (a == null || b == null){
			return ZERO;
		}
		BigDecimal aVal = toBigDecimal(a);
		BigDecimal bVal = toBigDecimal(b);
		return aVal.multiply(bVal).setScale(scale,roundingMode);
	}

	/***
	 * 乘法
	 * @param aVal 被乘数
	 * @param bVal 乘数
	 * @param scale 精度
	 * @return 乘法BigDecimal对象结果
	 */
	public static BigDecimal multiply(BigDecimal aVal, BigDecimal bVal,int scale) {
		if (aVal == null || aVal.compareTo(ZERO)==0 || bVal == null || bVal.compareTo(ZERO)==0){
			return ZERO;
		}
		return aVal.multiply(bVal).setScale(scale,BigDecimal.ROUND_HALF_UP);
	}

	/***
	 * 乘法
	 * @param aVal 被乘数
	 * @param bVal 乘数
	 * @param scale 精度
	 * @param roundingMode 进位方式
	 * @return 乘法BigDecimal对象结果
	 */
	public static BigDecimal multiply(BigDecimal aVal, BigDecimal bVal,int scale,int roundingMode) {
		if (aVal == null || aVal.compareTo(ZERO)==0 || bVal == null || bVal.compareTo(ZERO)==0){
			return ZERO;
		}
		return aVal.multiply(bVal).setScale(scale,roundingMode);
	}
	
	/***
	 * 加法
	 * @param a 被加数
	 * @param b 加数
	 * @param scale 精度
	 * @return 加法BigDecimal对象结果
	 */
	public static BigDecimal add(Object a, Object b,int scale) {
		BigDecimal aVal = toBigDecimal(a);
		BigDecimal bVal = toBigDecimal(b);
		return aVal.add(bVal).setScale(scale, BigDecimal.ROUND_HALF_UP);
	}

	/***
	 * 加法
	 * @param a 被加数
	 * @param b 加数
	 * @param scale 精度
	 * @param roundingMode 进位方式
	 * @return 加法BigDecimal对象结果
	 */
	public static BigDecimal add(Object a, Object b,int scale,int roundingMode) {
		BigDecimal aVal = toBigDecimal(a);
		BigDecimal bVal = toBigDecimal(b);
		return aVal.add(bVal).setScale(scale, roundingMode);
	}

	/***
	 * 加法
	 * @param aVal 被加数
	 * @param bVal 加数
	 * @param scale 精度
	 * @return 加法BigDecimal对象结果
	 */
	public static BigDecimal add(BigDecimal aVal, BigDecimal bVal,int scale) {
		return aVal.add(bVal).setScale(scale, BigDecimal.ROUND_HALF_UP);
	}

	/***
	 * 加法
	 * @param aVal 被加数
	 * @param bVal 加数
	 * @param scale 精度
	 * @param roundingMode 进位方式
	 * @return 加法BigDecimal对象结果
	 */
	public static BigDecimal add(BigDecimal aVal, BigDecimal bVal,int scale,int roundingMode) {
		return aVal.add(bVal).setScale(scale, roundingMode);
	}

	/***
	 * 减法
	 * @param a 被减数
	 * @param b 减数
	 * @param scale 精度
	 * @return 减法BigDecimal对象结果
	 */
	public static BigDecimal sub(Object a, Object b,int scale) {
		BigDecimal aVal = toBigDecimal(a);
		BigDecimal bVal = toBigDecimal(b);
		return aVal.subtract(bVal).setScale(scale, BigDecimal.ROUND_HALF_UP);
	}

	/***
	 * 减法
	 * @param a 被减数
	 * @param b 减数
	 * @param scale 精度
	 * @param roundingMode 进位方式
	 * @return 减法BigDecimal对象结果
	 */
	public static BigDecimal sub(Object a, Object b,int scale,int roundingMode) {
		BigDecimal aVal = toBigDecimal(a);
		BigDecimal bVal = toBigDecimal(b);
		return aVal.subtract(bVal).setScale(scale, roundingMode);
	}

	/***
	 * 减法
	 * @param aVal 被减数
	 * @param bVal 减数
	 * @param scale 精度
	 * @return 减法BigDecimal对象结果
	 */
	public static BigDecimal sub(BigDecimal aVal, BigDecimal bVal,int scale) {
		return aVal.subtract(bVal).setScale(scale, BigDecimal.ROUND_HALF_UP);
	}

	/***
	 * 减法
	 * @param aVal 被减数
	 * @param bVal 减数
	 * @param scale 精度
	 * @param roundingMode 进位方式
	 * @return 减法BigDecimal对象结果
	 */
	public static BigDecimal sub(BigDecimal aVal, BigDecimal bVal,int scale,int roundingMode) {
		return aVal.subtract(bVal).setScale(scale, roundingMode);
	}

	/**
	 * 加1
	 * @param a 被加数
	 * @param scale 精度
	 * @return BigDecimal对象结果
	 */
	public static BigDecimal addOne(Object a,int scale) {
		return add(a, ONE, scale);
	}

	/**
	 * 减1
	 * @param a 被减数
	 * @param scale 精度
	 * @return BigDecimal对象结果
	 */
	public static BigDecimal subOne(Object a,int scale) {
		return sub(a, ONE, scale);
	}
	
	/**
	 * 数值转大写
	 * @param n 数值
	 * @return 数值大写String对象
	 */
	public static String digitCapital(double n) {
	    String fraction[] = {"角", "分"};
	    String digit[] = {"零", "壹", "贰", "叁", "肆", "伍", "陆", "柒", "捌", "玖"};
	    String unit[][] = {{"元", "万", "亿"}, {"", "拾", "佰", "仟"}};

	    String head = n < 0 ? "负" : "";
	    // 如果是负数取绝对值
	    n = Math.abs(n);
	    String s = "";
	    BigDecimal bigDecimal = new BigDecimal(Double.valueOf(n).toString());
	    String nStr = bigDecimal.toPlainString();
	    // 小数部分
	    String[] split = nStr.split("\\.");
	    if (split.length > 1) {
	        // 小数点为特殊符号，在分割时需进行转义
	        String decimalStr = split[1];
	        if (decimalStr.length() > 2) {
	            decimalStr = decimalStr.substring(0, 2);
	        }
	        // 将小数部分转换为整数
	        Integer integer = Integer.valueOf(decimalStr);
	        String p = "";
	        for (int i = 0; i < decimalStr.length() && i < fraction.length; i++) {
	            p = digit[integer % 10] + fraction[decimalStr.length() - i - 1] + p;
	            integer = integer / 10;
	        }
	        s = p.replaceAll("(零.)+", "") + s;
	    }
	    if (s.length() < 1) {
	        s = "整";
	    }
	    int integerPart = (int)Math.floor(n);
	    // 整数部分
	    for (int i = 0; i < unit[0].length && integerPart > 0; i++) {
	        String p = "";
	        for (int j = 0; j < unit[1].length && n > 0; j++) {
	            p = digit[integerPart % 10] + unit[1][j] + p;
	            integerPart = integerPart / 10;
	        }
	        s = p.replaceAll("(零.)*零$", "").replaceAll("^$", "零") + unit[0][i] + s;
	    }
	    return head + s.replaceAll("(零.)*零元", "元").replaceFirst("(零.)+", "").replaceAll("(零.)+", "零").replaceAll("^整$", "零元整");
	}

	/**
	 * 向上取整
	 * @param aVal 数值
	 * @param scale 精度
	 * @return BigDecimal对象结果
	 */
	public static BigDecimal ceil(double aVal,int scale) {
		aVal = Math.ceil(aVal);
		BigDecimal aVal1=BigDecimal.valueOf(aVal);
		aVal1 = aVal1.setScale(scale, BigDecimal.ROUND_HALF_UP);
		return aVal1;
	}

	/**
	 * 向下取整
	 * @param aVal 数值
	 * @param scale 精度
	 * @return BigDecimal对象结果
	 */
	public static BigDecimal floor(double aVal,int scale) {
		aVal = Math.floor(aVal);
		BigDecimal aVal1=BigDecimal.valueOf(aVal);
		aVal1 = aVal1.setScale(scale, BigDecimal.ROUND_HALF_UP);
		return aVal1;
	}

	/**
	 * 四舍五入
	 * @param obj Object对象
	 * @param scale 精度
	 * @return BigDecimal对象结果
	 */
	public static BigDecimal toBigDecimal(Object obj,int scale) {
		BigDecimal aVal1 = toBigDecimal(obj);
		aVal1 = aVal1.setScale(scale, BigDecimal.ROUND_HALF_UP);
		return aVal1;
	}

	/**
	 * 对象转数值字符串格式
	 * @param d Object对象
	 * @return 字符串对象结果
	 */
	public static String formatDouble(Object d) {
        NumberFormat nf = NumberFormat.getInstance();
        //设置保留多少位小数
        nf.setMaximumFractionDigits(20);
         // 取消科学计数法
        nf.setGroupingUsed(false);
        //返回结果
        return nf.format(d);
    }
}
